Struct HumanFloatCount

struct HumanFloatCount(195)

Formats counts for human readability using commas for floats

Implementations

impl Debug for HumanFloatCount

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Display for HumanFloatCount

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for HumanFloatCount

impl RefUnwindSafe for HumanFloatCount

impl Send for HumanFloatCount

impl Sync for HumanFloatCount

impl Unpin for HumanFloatCount

impl UnsafeUnpin for HumanFloatCount

impl UnwindSafe for HumanFloatCount

impl<T> Any for HumanFloatCount

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for HumanFloatCount

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for HumanFloatCount

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for HumanFloatCount

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToString for HumanFloatCount

fn to_string(self: &Self) -> String

impl<T, U> Into for HumanFloatCount

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for HumanFloatCount

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for HumanFloatCount

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>