Struct ProgressDrawTarget

struct ProgressDrawTarget { ... }

Target for draw operations

This tells a ProgressBar or a MultiProgress object where to paint to. The draw target is a stateful wrapper over a drawing destination and internally optimizes how often the state is painted to the output device.

Implementations

impl ProgressDrawTarget

fn stdout() -> Self

Draw to a buffered stdout terminal at a max of 20 times a second.

For more information see ProgressDrawTarget::term.

fn stderr() -> Self

Draw to a buffered stderr terminal at a max of 20 times a second.

This is the default draw target for progress bars. For more information see ProgressDrawTarget::term.

fn stdout_with_hz(refresh_rate: u8) -> Self

Draw to a buffered stdout terminal at a max of refresh_rate times a second.

For more information see ProgressDrawTarget::term.

fn stderr_with_hz(refresh_rate: u8) -> Self

Draw to a buffered stderr terminal at a max of refresh_rate times a second.

For more information see ProgressDrawTarget::term.

fn term(term: Term, refresh_rate: u8) -> Self

Draw to a terminal, with a specific refresh rate.

Progress bars are by default drawn to terminals however if the terminal is not user attended the entire progress bar will be hidden. This is done so that piping to a file will not produce useless escape codes in that file.

Will panic if refresh_rate is 0.

fn term_like(term_like: Box<dyn TermLike>) -> Self

Draw to a boxed object that implements the TermLike trait.

fn term_like_with_hz(term_like: Box<dyn TermLike>, refresh_rate: u8) -> Self

Draw to a boxed object that implements the TermLike trait, with a specific refresh rate.

fn hidden() -> Self

A hidden draw target.

This forces a progress bar to be not rendered at all.

fn is_hidden(self: &Self) -> bool

Returns true if the draw target is hidden.

This is internally used in progress bars to figure out if overhead from drawing can be prevented.

impl Debug for ProgressDrawTarget

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Freeze for ProgressDrawTarget

impl RefUnwindSafe for ProgressDrawTarget

impl Send for ProgressDrawTarget

impl Sync for ProgressDrawTarget

impl Unpin for ProgressDrawTarget

impl UnsafeUnpin for ProgressDrawTarget

impl UnwindSafe for ProgressDrawTarget

impl<T> Any for ProgressDrawTarget

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for ProgressDrawTarget

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for ProgressDrawTarget

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for ProgressDrawTarget

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for ProgressDrawTarget

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for ProgressDrawTarget

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for ProgressDrawTarget

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>