Trait ProgressIterator
trait ProgressIterator
where
Self: Sized + Iterator
Wraps an iterator to display its progress.
Required Methods
fn progress_with(self: Self, progress: ProgressBar) -> ProgressBarIter<Self>Wrap an iterator with a custom progress bar.
Provided Methods
fn try_progress(self: Self) -> Option<ProgressBarIter<Self>>Wrap an iterator with default styling. Uses [
Iterator::size_hint()] to get length. ReturnsSome(..)only ifsize_hint.1isSome. If you want to create a progress bar even ifsize_hint.1returnsNoneuseprogress_count()orprogress_with()instead.fn progress(self: Self) -> ProgressBarIter<Self> where Self: ExactSizeIteratorWrap an iterator with default styling.
fn progress_count(self: Self, len: u64) -> ProgressBarIter<Self>Wrap an iterator with an explicit element count.
fn progress_with_style(self: Self, style: ProgressStyle) -> ProgressBarIter<Self> where Self: ExactSizeIteratorWrap an iterator with a progress bar and style it.
Implementors
impl<S, T> ProgressIterator for ProgressBarIter<T>impl<S, T: Iterator<Item = S>> ProgressIterator for T